#!/bin/bash
# ==== Variables
config_path="./bin/config.txt"

# ==== Functions
function search_array() {
    index=0
    while [ "$index" -lt "${#config_key[@]}" ]; do
        if [ "${config_key[$index]}" = "$1" ]; then
            echo ${config_value[$index]}
            return
        fi
        let "index++"
    done
    echo ""
}

# ==== Looking for configuration file
if [ ! -r ./bin/config.txt ];then
	python ./bin/create_config.py	# Creating the configuration file
	if [ "${?}" -ne "0" ] ; then	
		exit
	fi
fi


# ==== Parsing configuration file
i=0
while read line; do
if [[ "$line" =~ ^[^#]*= ]]; then
        config_key[i]=`echo $line | cut -d'=' -f 1`
        config_value[i]=`echo $line | cut -d'=' -f 2-`
        ((i++))
fi
done < $config_path

#echo "total array elements: ${#name[@]}"
#echo "name[0]: ${name[0]}"
#echo "value[0]: ${value[0]}"
#echo "name[1]: ${name[1]}"
#echo "value[1]: ${value[1]}"
#echo "key array: ${config_key[@]}"
#echo "value array: ${config_value[@]}"

# ==== Evaluating command line
PYTHON=$(search_array "PYTHON")		# getting the python path
if [ -z "$PYTHON" ]; then
	echo -e "the key \"$value\" is not in $config_path"
	exit 1
fi

if [ $# -eq 0 ]; then				# Launch CellTracking
	./bin/CellTracking_main.sh
elif ([[ $1 == "--align" ]] && [ $# -eq 2 ]); then		# Align
	eval "$PYTHON ./bin/CT_alignment.py $2"
elif [[ $1 == "--mosaic" ]]; then	# Mosaic
	eval "$PYTHON ./bin/CT_views.py $2"
else
	echo "USAGE: ./CellTracking.sh [option] [params]"
	echo "No option specified: Load the main interface"
	echo ""
	echo "Options: --align --mosaic"
	echo "--align ./path_to_alignment_file"
	echo "--mosaic ./path_to_mosaic_file"
fi
