#! /usr/bin/env python
# _*-coding:utf-8-*-
# Modifié par PPPP, GPLv3+
# Script qui gère le tracking depuis une source vidéo donnée
# mod_tracking v0.1.8/AntsTracking0.1.8

print "Based on the OpenCV Python version of lkdemo"
print "Modified for AntsTracking (2012) and CellTracking (2012)"

# ========== Importations
import cv

# ========== Fonctions   
def tracking_init() :	# Fonction qui initialise les variables nécessaires au tracking et les stocke sous la forme d'un dictionnaire
	data = {'win_size' : 10, 'MAX_COUNT' : 500, 		
	'image' : None,
	'pt' : None,
	'pts' : [],
	'add_remove_pt' : False,
	'flags' : 0,
	'night_mode' : False,
	'need_to_init' : False }	# Variables globales, puis d'autres			
	return data

def tracking(frame, data) :		# La fonction principale
        if data['image'] is None:
            # create the images we need
            data['image'] = cv.CreateImage(cv.GetSize(frame), 8, 3)	# Création de l'image
            data['image'].origin = frame.origin				# ajout de propriété à image
            data['grey'] = cv.CreateImage(cv.GetSize(frame), 8, 1)
            data['prev_grey'] = cv.CreateImage(cv.GetSize(frame), 8, 1)
            data['pyramid'] = cv.CreateImage(cv.GetSize (frame), 8, 1)
            data['prev_pyramid'] = cv.CreateImage (cv.GetSize (frame), 8, 1)
            data['features'] = []
        cv.Copy(frame, data['image'])	 # copy the frame to image, so we can draw on it
        cv.CvtColor(data['image'], data['grey'], cv.CV_BGR2GRAY)         # create a grey version of the image

        if data['need_to_init'] :	 # we want to search all the good points
            # create the wanted images
            eig = cv.CreateImage (cv.GetSize (data['grey']), 32, 1)
            temp = cv.CreateImage (cv.GetSize (data['grey']), 32, 1)
            # the default parameters
            quality = 0.05
            min_distance = 10
            # search the good points
            data['features'] = cv.GoodFeaturesToTrack (
                data['grey'], eig, temp,
                data['MAX_COUNT'],
                quality, min_distance, None, 3, 0, 0.04)

            # refine the corner locations
            data['features'] = cv.FindCornerSubPix (
                data['grey'],
                data['features'],
                (data['win_size'], data['win_size']),  (-1, -1),
                (cv.CV_TERMCRIT_ITER | cv.CV_TERMCRIT_EPS, 20, 0.03))

        elif data['features'] != []:
            # we have points, so display them

            # calculate the optical flow
            data['features'], status, track_error = cv.CalcOpticalFlowPyrLK (
                data['prev_grey'], data['grey'], data['prev_pyramid'], data['pyramid'],
                data['features'],
                (data['win_size'], data['win_size']), 3,
                (cv.CV_TERMCRIT_ITER|cv.CV_TERMCRIT_EPS, 20, 0.03),
                data['flags'])

            # set back the points we keep
            data['features'] = [ p for (st,p) in zip(status, data['features']) if st]

            if data['add_remove_pt']:
                # we have a point to add, so see if it is close to
                # another one. If yes, don't use it
                def ptptdist(p0, p1):
                    dx = p0[0] - p1[0]
                    dy = p0[1] - p1[1]
                    return dx**2 + dy**2
                if min([ ptptdist(data['pt'], p) for p in data['features'] ]) < 25:
                    # too close
                    data['add_remove_pt'] = 0
	if data['pts'] != [] :
		for i in data['pts'] :
			data['features'] += cv.FindCornerSubPix (
	                data['grey'],
        	        [i],
        	        (data['win_size'], data['win_size']),  (-1, -1),
        	        (cv.CV_TERMCRIT_ITER | cv.CV_TERMCRIT_EPS,
        	        20, 0.03))
        	data['pts'] = []
        	
        if data['add_remove_pt']:
            # we want to add a point
            # refine this corner location and append it to 'features'

            data['features'] += cv.FindCornerSubPix (
                data['grey'],
                [data['pt']],
                (data['win_size'], data['win_size']),  (-1, -1),
                (cv.CV_TERMCRIT_ITER | cv.CV_TERMCRIT_EPS,
                20, 0.03))
            # we are no longer in "add_remove_pt" mode
            data['add_remove_pt'] = False

        # swapping
        data['prev_grey'], data['grey'] = data['grey'], data['prev_grey']
        data['prev_pyramid'], data['pyramid'] = data['pyramid'], data['prev_pyramid']
        data['need_to_init'] = False
        
