# Functions that manage the creation of a polygon object and its edition.
# This class inherits from the main Cell class (or the Polygon one

import imp, numpy
cells = imp.load_source('ct_Cells', './modules/modules_core/ct_Cells.py')


name = 'polygon'
Cell = cells.Cell

def get_name() :
	return name

class CellWrapper(Cell) :
	def __init__(self, movie, nb_image, label, use_gtk) :
		self.editable = True
		self.type = 'polygon'
		Cell.__init__(self, movie, nb_image, label, use_gtk)
		
		#####self.center_rect = (-1,-1)
		self.tmp_poly = []
		self.reference_size = 8
		
		# TMP?
		self.threshold = 80
		self.font_tmp = None

	def add_point(self, point, frame) :
		"""Adds the new point to the polygon"""
		# See if the point is closing the polygon
		if (len(self.tmp_poly) >=2 and self.distance(point, self.tmp_poly[0]) < self.threshold) :
			self.close = True
			self.color = self.c_color
			self.add_polygon(self.tmp_poly, frame)
			self.tmp_poly = []
		else :
			self.tmp_poly.append(point)
		return self.close
		
	def get_reference_poly(self, i) :
		"""Returns the reference polygon"""
		r_size = self.reference_size
		poly = self.get_poly(i)
		(x_g,y_g) = self.get_center(i)
		
		poly2 = []
		for point in poly :
			d = ( (point[0]-x_g)**2 + (point[1]-y_g)**2 )**.5   # Center--Point distance
			# Replace by vectorial calculus?
			x = point[0] - (x_g - point[0])*float(r_size)/d       # Coordinate of point of reference
			y = point[1] - (y_g - point[1])*float(r_size)/d
			poly2.append((x,y))
		
		return poly2
	
	def get_center(self, i) :
		"""Returns gravity center"""
		p = self.get_poly(i)
		g = numpy.asarray(p).mean(axis=0)
		return tuple(g)
	
	def draw_polygon(self, image, i, cv, debug=False) :
		self.cv = cv
		if self.tmp_poly == [] :
			Cell.draw_polygon(self, image, i, cv, debug)
		else :
			if self.font_tmp == None :
				font = cv.InitFont(cv.CV_FONT_HERSHEY_PLAIN, 1, 1) 	# Initializing font
				self.font_tmp = font
			image = self.draw_open_polygon(image, i)
		return image

	def move(self,pt, im) :
		"""Functions that moves a point"""
		p = self.get_poly(im)		# Get the polygon
		idx = self.polygon.get_index(im)	# Gets the index of the image
		
		self.poly_point = []
		#print "IDX:" + str(idx)	# Debug
		if idx == None :	# If the key_poly we are editing doesn't exists
			#print "Creating new frame"	# Debug
			self.polygon.create_frame(im)
			idx = self.polygon.get_index(im)	# Gets the index of the image
		
		# Determining which point to move
		for i in range(len(p)) : # Determine what point to move
			d = self.distance(p[i], pt)# Determine the distance
			if d < self.threshold :
				self.poly_point = (p[i], i, idx, im)	# [Coordinates, number of the coordinate, frame_key]
				break
		if self.poly_point != [] :
			#self.big_points = (self.poly_point[0], self.color)
			self.big_points = (self.polygon.get_object(i)[self.poly_point[1]], self.color)
		return True
		
	def move2(self, point_m, i) :
		"""Function that really does the movement and updates everything"""
		ret = False
		if point_m == (-1, -1) :
			self.poly_point = False
			self.moving = False
		if self.poly_point :	# Do something only if a point has really been clicked
			point_p = self.poly_point		# Get polygon's point location
			pol = self.polygon.get_object(i)
			pol[point_p[1]] = point_m	# Get polygon object and replace it
			self.polygon.add_object(pol, i)
			self.big_points = (point_m,self.color)
			ret = True
		if ret :
			self.moving = True
		return ret

	def draw_open_polygon(self, image, i) :
		p=self.tmp_poly
		if len(p) >=2 :
			#print p
			pp = []
			for j in p :
				pp.append(tuple(j))
			self.cv.PolyLine(image, [pp], self.close, self.color, 1)
		for point in p :
			self.cv.Circle(image, tuple(point), 3, self.color, -1)
		self.cv.PutText(image, str(self.label), (p[0][0], p[0][1]), self.font_tmp, self.color)
		return image
		
	def get_type(self) :
		return self.type
		
	def distance(self, p1, p2) :
		d = (p1[0]-p2[0])**2 + (p1[1]-p2[1])**2
		return d
	def get_save_dict(self) :
		dic = Cell.get_save_dict(self)
		dic['editable'] = {'desc' : 'if the cell is editable', 'value' : self.editable}
		dic['reference_size'] = {'desc' : 'The size of the reference around the polygon', 'value' : self.reference_size}
		dic['type'] = {'desc' : 'Type of the cell, which define the CellWrapper to use', 'value' : self.type}
		return dic
		
	def set_save_dict(self, dic) :
		Cell.set_save_dict(self, dic)
		self.editable = dic['editable']['value']
		self.type = dic['type']['value']
		self.reference_size = dic['reference_size']['value']
	
