# Functions that manage the creation of a polygon object and its edition.
# This class inherits from the main Cell class (or the Polygon one

import imp
cells = imp.load_source('ct_Cells', './modules/modules_core/ct_Cells.py')

name = 'rectangle'
Cell = cells.Cell

def get_name() :
	return name

class CellWrapper(Cell) :
	def __init__(self, movie, nb_image, label, use_gtk) :
		self.editable = True
		self.type = 'rectangle'
		Cell.__init__(self, movie, nb_image, label, use_gtk)
		
		self.center_rect = (-1,-1)
		self.tmp_poly = []
		self.reference_size = 15
	def add_point(self, point, frame) :
		if self.center_rect == (-1,-1) :	# initializing the center of the rect
			self.center_rect = point
		else :	# Add 4 points to the list, close polygon
			self.tmp_poly = self.calc_rect(self.center_rect, point)
			self.close = True
			self.color = self.c_color
			self.add_polygon(self.tmp_poly, frame)
			self.center_rect = (-1,-1)
		return self.close
	def draw_polygon(self, image, i, cv, debug=False) :
		self.cv = cv
		if self.center_rect == (-1,-1) :
			image = Cell.draw_polygon(self, image, i, cv, debug)
			
			if not self.deleted and i >= self.start and i <= self.stop : # Inherited feature : draw polygon center
				if self.get_poly(i) != []:
					image=self.draw_center(image, i)
		else :
			image=self.draw_center_tmp(image, i)
		return image
	
	def get_reference_poly(self, i) :
		r = self.reference_size
		c = self.get_center(i)
		p = self.get_poly(i)
		pt = (p[0][0]-r, p[0][1]-r)
		return self.calc_rect(c, pt)

	def get_center(self, i) :
		"""Returns the center of a RECTANGLE given with point order
		given in the convention defined in the calc_rect function"""
		p = self.get_poly(i)
		c_x = int(p[0][0]+0.5*(p[1][0]-p[0][0]))
		c_y = int(p[2][1]+0.5*(p[1][1]-p[2][1]))
		return (c_x,c_y)	

	def calc_rect(self, c, pt) :
		"""Function that returns a rectangle centered on c and with pt
		as a vertex"""
		c_x = c[0]
		c_y = c[1]
		dx = abs(pt[0]-c_x)
		dy = abs(pt[1]-c_y)
		return [(c_x-dx,c_y-dy),(c_x+dx,c_y-dy),(c_x+dx,c_y+dy),(c_x-dx,c_y+dy)]

	def move(self,pt, im) :
		"""Functions that moves a point"""
		p = self.get_poly(im)		# Get the polygon
		idx = self.polygon.get_index(im)	# Gets the index of the image
		
		self.poly_point = []
		#print "IDX:" + str(idx)	# Debug
		if idx == None :	# If the key_poly we are editing doesn't exists
			#print "Creating new frame"	# Debug
			self.polygon.create_frame(im)
			idx = self.polygon.get_index(im)	# Gets the index of the image
		
		# ==== Determining which point to move
		if self.distance(self.get_center(im), pt) < self.threshold :
			self.poly_point = (self.center_rect, 'rect_center', idx, im)	# [Coordinates, number of the coordinate, frame_key]
			self.big_points = (self.get_center(im), self.color)		# EXP printing
		else :
			for i in range(len(p)) : # Determine what point to move
				d = self.distance(p[i], pt)# Determine the distance
				if d < self.threshold :
					self.poly_point = (p[i], i, idx, im)	# [Coordinates, number of the coordinate, frame_key]
					self.big_points = (self.polygon.get_object(im)[self.poly_point[1]], self.color)		# EXP printing
					break
		if self.poly_point != [] :
			pass
			#self.big_points = (self.poly_point[0], self.color)
			#if self.poly_point[1] == 'rect_center' :
			#	self.big_points = (self.get_center(im), self.color)
			#self.big_points = (self.key_poly[self.poly_point[2]][self.poly_point[1]], self.color)
		return True	
		
	def move2(self, point_m, i) :
		"""Function that really does the movement and updates everything"""
		ret = False
		if point_m == (-1, -1) :
			self.poly_point = False
			self.moving = False
		if self.poly_point :	# Do something only if a point has really been clicked
			point_p = self.poly_point		# Get polygon's point location
			if point_p[1] == 'rect_center' :
				center = self.get_center(i)
				dx = point_m[0] - center[0]
				dy = point_m[1] - center[1]
				out_p = []
				#for p in self.key_poly[point_p[2]] :	# Move polygon
				for p in self.polygon.get_object(i) :	# Move polygon
					out_p.append((p[0]+dx,p[1]+dy))
				self.polygon.add_object(out_p, i)	
				#self.key_poly[point_p[2]] = out_p
				self.big_points = (center,self.color)
				ret = True
			else :
				#old_pt = self.key_poly[point_p[2]][point_p[1]]
				old_pt = self.polygon.get_object(i)[point_p[1]]
				[c_x, c_y] = self.get_center(i)
				dx = abs(point_m[0] - c_x)
				dy = abs(point_m[1] - c_y)
				#self.key_poly[point_p[2]] = [(c_x-dx,c_y-dy),(c_x+dx,c_y-dy),(c_x+dx,c_y+dy),(c_x-dx,c_y+dy)]
				self.polygon.add_object([(c_x-dx,c_y-dy),(c_x+dx,c_y-dy),(c_x+dx,c_y+dy),(c_x-dx,c_y+dy)], i)
				self.big_points = (point_m,self.color)
				ret = True
		if ret :
			self.moving = True
		return ret	

	def draw_center_tmp(self, image, i) :
		"""Draws the center of a rectangle"""
		c = self.center_rect
		self.cv.Circle(image, c, 5, self.color, -1)
		return image

	def calc_rect(self, c, pt) :
		"""Function that returns a rectangle centered on c and with pt
		as a vertex"""
		c_x = c[0]
		c_y = c[1]
		dx = abs(pt[0]-c_x)
		dy = abs(pt[1]-c_y)
		return [(c_x-dx,c_y-dy),(c_x+dx,c_y-dy),(c_x+dx,c_y+dy),(c_x-dx,c_y+dy)]
	
	def distance(self, p1, p2) :
		d = (p1[0]-p2[0])**2 + (p1[1]-p2[1])**2
		return d	
		
	def get_type(self) :
		return self.type

	def get_save_dict(self) :
		dic = Cell.get_save_dict(self)
		dic['editable'] = {'desc' : 'if the cell is editable', 'value' : self.editable}
		dic['reference_size'] = {'desc' : 'The size of the reference around the polygon', 'value' : self.reference_size}
		dic['type'] = {'desc' : 'Type of the cell, which define the CellWrapper to use', 'value' : self.type}
		return dic
		
	def set_save_dict(self, dic) :
		Cell.set_save_dict(self, dic)
		self.editable = dic['editable']['value']
		self.type = dic['type']['value']
		self.reference_size = dic['reference_size']['value']
