import time, copy, imp, cv
import numpy, cv2
mod_phases = imp.load_source('mod_phases', './modules/modules_core/mod_phases.py')
mod_fluorescence = imp.load_source('mod_fluorescence', './modules/modules_treatment/mod_fluorescence.py')
ct_TimeTrack = imp.load_source('ct_TimeTrack', './modules/modules_core/ct_TimeTrack.py')

list_wrappers = [('mod_cell_polygon', './modules/modules_cell/mod_cell_polygon.py'),
				('mod_cell_rectangle', './modules/modules_cell/mod_cell_rectangle.py'),
				('mod_cell_grid', './modules/modules_cell/mod_cell_grid.py')]


class Cell :
	"""Polygon and its basic properties"""
	#def __init__(self, nb_im, label, font, poly_mode) :
	def __init__(self, movie, nb_im, label, use_gtk) :
		self.cv = cv
		self.movie = movie
		self.use_gtk = use_gtk
		if use_gtk :
			mod = imp.load_source('ct_Cells_GTK', './modules/modules_core/ct_Cells_GTK.py')
			self.gui = mod.Cell_GTK(self)
		
		# Modules	
		self.phases = mod_phases.Phases(self, movie)
		self.fluorescence = mod_fluorescence.Fluorescence(self, self.movie)
		self.polygon = ct_TimeTrack.TimeTrack(0, nb_im)
		self.rectangle = ct_TimeTrack.TimeTrack(0, nb_im)
			
		self.font_init = False
		self.deleted = False
		self.label = label
		#self.max_im = nb_im
		self.start = 0
		self.stop = nb_im
		####self.key_poly = []	# list of the polygons
		####self.key_frames = []	# list of the associated frames
		self.close = False
		self.selected = False
		
		self.o_color = 	(0, 255, 0, 255)	# Color for an open poly (previously pygame.Color)
		self.c_color =	(255, 0, 0, 255)	# Color of a closed poly
		self.r_color =	(255, 255, 0, 255)	# Color of a reference poly
		self.s_color =	(0, 0, 255, 255)	# Color of a selected poly
		self.color = self.o_color			# Nowadays' color
		#self.rect = []
		self.threshold = 80	# Maximum distance/tolerance/radius to close a polygon
		
		# Buffers for point printing when moving a point
		self.big_points = []
		self.small_points = []
		self.moving = False
	def get_save_dict(self) :
		dic = {}
		dic['label'] = {'desc' : 'The label of the cell', 'value' : self.get_label()}
		dic['start'] = {'desc' : 'Beginning time of the cell', 'value' : self.start}
		dic['stop'] = {'desc' : 'Ending time of the cell', 'value' : self.stop}
		dic['selected'] = {'desc' : 'if the cell is selected or excluded', 'value' : self.is_selected()}
		dic['deleted'] = {'desc' : 'if the cell is deleted', 'value' : self.deleted}
		dic['close'] = {'desc' : 'if the polygon is closed', 'value' : self.close}
		dic['phases'] = {'desc' : 'Phases data', 'value' : self.phases.get_save_dict()}
		dic['polygon'] = {'desc' : 'Information about the moving polygon defining the cell', 'value' : self.polygon.get_save_dict()}
		dic['rectangle'] = {'desc' : 'Information about the moving rectangle surrounding the cell', 'value' : self.rectangle.get_save_dict()}
		#dic['color'] = {'desc' : 'Color of the polygon', 'value' : self.color}
		return dic
		
	def set_save_dict(self, dic) :
		self.label = dic['label']['value']
		self.start = dic['start']['value']
		self.stop = dic['stop']['value']
		self.selected = dic['selected']['value']
		self.deleted = dic['deleted']['value']
		self.phases.set_save_dict(dic['phases']['value'])
		self.polygon.set_save_dict(dic['polygon']['value'])
		self.rectangle.set_save_dict(dic['rectangle']['value'])
		#self.color = dic['color']['value']
		self.color = self.c_color
		self.close = dic['close']['value']
		#print self.selected
	
	def get_label(self) :
		return self.label
	def is_selected(self) :
		return self.selected
	def set_selected(self, state) :
		self.selected = state
	
	def set_label(self, l) :
		self.label = l
		
	def set_frame(self, i) :
		"""Saves the given index as the index of the current frame.
		
		.. warning::
			At that time, no verification is performed, and you must check that the input index is a valid index
			
		:param i: The index of the frame
		:type i: int
		"""
		self.frame = int(i)

	def select(self, f) :
		"""f=1 selects the polygon
		OR simply change the selection state ?"""
		if f :
			self.selected = True
		else :
			self.selected = False	
		self.ch_color()	# Updating color
	def ch_color(self) :
		if self.selected :
			self.color = self.s_color
		elif self.close :
			self.color = self.c_color
		else :
			self.color = self.o_color
			
			
	def add_polygon(self, poly, frame) :
		self.polygon.add_object(poly, frame)
		# See below for the generation of the rectangle
		"""# Adjust opening/closing time
		if frame < self.start :
			self.start = frame
		elif frame > self.stop :
			self.stop = frame

		# Get the index of the key_poly or create it
		if frame in self.key_frames :
			#print '-- getting frame index'
			i = self.key_frames.index(frame)
		elif self.key_frames==[] :
			#print '-- creating first frame'
			self.key_frames.append(frame)
			self.key_poly.append([])
			self.rect.append([])
			i=0		
		else :
			#print '-- creating new frame'
			i=self.create_frame(frame)
		#print self.key_frames
		
		# Adding the polygon at the right place
		self.key_poly[i] = poly"""
		
		# Generating the rect
		x_list = []
		y_list = []
		for (x,y) in poly :
			x_list.append(x)
			y_list.append(y)
		x_M = max(x_list)
		x_m = min(x_list)
		y_M = max(y_list)
		y_m = min(y_list)
		rect = [x_m, y_m, x_M-x_m, y_M-y_m]
		self.rectangle.add_object(rect, frame)
	
		
	def create_frame(self, im) :
		"""Function that inserts a frame by copying the previous one
		That function doesn't check if the frame already exists
		it MUST have been checked before !"""
		"""i=-1
		if im > self.key_frames[-1] :
			self.key_frames.append(im)
			self.key_poly.append(copy.copy(self.key_poly[-1]))	# /!\ Don't insert blank frame, COPY the previ
			self.rect.append(copy.copy(self.rect[-1]))
			i = len(self.key_frames)-1
			#print "frame " + str(im) +" inserted at position " + str(len(self.key_frames)-1)	# Debug
		elif im < self.key_frames[0] :
			self.key_frames.insert(0,im)
			self.key_poly.insert(0, copy.copy(self.key_poly[0]))	# /!\ Don't insert blank frame, COPY the previ
			self.rect.insert(0, copy.copy(self.rect[-1]))
			#print "frame " + str(im) +" inserted at position " + str(len(self.key_frames)-1)	# Debug
			i=0
		else :
			i=1
			while i <= len(self.key_frames) :
				if self.key_frames==[] or self.key_frames[i] > im :	# Surrounding frames : [i-1, i]
					poly = self.interpolate(i, im)		# Interpolate poly
					self.key_frames.insert(i,im)		# Register frames
					self.key_poly.insert(i, copy.copy(poly))
					self.rect.insert(i, []) 		# Save poly
					self.get_rect(im, True)			# Regenerate rect
					#print "frame " + str(im) +" inserted at position " + str(i)	# Debug
					break
				i +=1
		if len(self.key_frames) != len(self.key_poly) :
			print "INCOHERENT SIZES"
		return i"""
	
	def export_frame(self, i, paths, rect=None) :
		if i>=self.start and i<=self.stop :
			c = self.get_cell(i, depth=16, col_channels=1, rect=rect)					# Getting OpenCV image
			cv.SaveImage(paths[0], c[0])			
			#cv.SaveImage(paths[1], c[1])	# Masks
			#cv.SaveImage(paths[2], c[2])
	
	def get_cell(self, i, channel=None, depth=8, col_channels=1, fmt='cv', rect=None) :
		"""Function that returns the current cell image, as a tuple
		The image with the surrounding rectangle as a copy of the original image
		A mask representing the cell
		A mask representing the reference
		All in OpenCV format"""
		
		# getting everything
		cv = self.cv
		im_tmp = self.movie.get_frame(i, channel, depth, col_channels, fmt) 		# The input image
		if fmt == 'cv' :
			im=cv.CreateImage(cv.GetSize(im_tmp), im_tmp.depth, im_tmp.channels)
			cv.Copy(im_tmp, im)
		
		cel_poly = self.get_poly(i)										# Cell polygon
		ref_poly = self.get_reference_poly(i)

		# centroid
		centroid = numpy.array(cel_poly).mean(axis=0)
		
		# ROI
		if rect != None :
			#print "recalcul"		# DEBUG
			ref_rect = (int(centroid[0] - rect[0]/2), int(centroid[1] - rect[1]/2), rect[0], rect[1])
		else :	
			#print "norecalcul"		# DEBUG
			ref_rect = numpy.array(self.get_rect(i, with_reference=True), dtype=numpy.int)
			ref_rect = numpy.maximum(ref_rect,0)	# Put negative values to zero
		
		
		# Computing mask data
		x0 = ref_rect[0]
		y0 = ref_rect[1]
		cel_poly_converted = []
		for p in cel_poly :
			cel_poly_converted.append((p[0]-x0, p[1]-y0))
		ref_poly_converted = []
		for p in ref_poly :
			ref_poly_converted.append((p[0]-x0, p[1]-y0))
		
		
		if fmt=='cv2' :
			out = im_tmp[ref_rect[1]:(ref_rect[1]+ref_rect[3]), ref_rect[0]:(ref_rect[0]+ref_rect[2])]
			#print [out.shape, ref_rect]		# DEBUG
			cel_mask = numpy.zeros(out.shape)
			ref_mask = numpy.zeros(out.shape)
			#cv2.fillPoly(cel_mask, [numpy.asarray(cel_poly_converted)], 255)	#cv2.fillPoly(cel_mask, numpy.array([cel_poly_converted], dtype='int32'), 255)
			#cv2.fillPoly(ref_mask, [numpy.asarray(ref_poly_converted)], 255)	#cv2.fillPoly(ref_mask, numpy.array([ref_poly_converted], dtype='int32'), 255)
			cv2.fillPoly(cel_mask, numpy.array([cel_poly_converted], dtype='int32'), 255)
			cv2.fillPoly(ref_mask, numpy.array([ref_poly_converted], dtype='int32'), 255)
			
			ref_mask = ref_mask - cel_mask
			return (out, cel_mask, ref_mask)
		
		# Output image
		cv.SetImageROI(im, tuple(ref_rect))
			
		# Cell mask
		cel_mask = cv.CreateImage(cv.GetSize(im), 8, 1)
		cv.SetZero(cel_mask)
		cv.FillPoly(cel_mask, [cel_poly_converted], 255)
		
		# Cell reference
		ref_mask = cv.CreateImage(cv.GetSize(im), 8, 1)
		cv.SetZero(ref_mask)
		cv.FillPoly(ref_mask, [ref_poly_converted], 255)
		cv.Sub(ref_mask, cel_mask, ref_mask)
		
		"""import pylab, numpy
		pylab.figure()
		pylab.imshow(numpy.asarray(cv.GetMat(im)))
		pylab.figure()
		pylab.imshow(numpy.asarray(cv.GetMat(cel_mask)))
		pylab.figure()
		pylab.imshow(numpy.asarray(cv.GetMat(ref_mask)))
		pylab.show()"""
		
		return (im, cel_mask, ref_mask)
		
	def intersection(self, point, i) :
		"""Sees if the click is in the polygon"""
		res = False
		if not self.deleted and i >= self.start and i <= self.stop :
			#r=self.get_rect(i)
			#tt = self.threshold_r
			#r = [r[0]-tt, r[1]-tt, r[2]+tt, r[3]+2*tt]
			p = self.get_poly(i)
			res = -1
			if p != [] :
				#res = self.cv.PointPolygonTest(self.get_poly(i), point, 0)
				res = self.cv.PointPolygonTest(self.get_reference_poly(i), point, 0)
			if res >= 0 :
				res = True
			else : res = False
		return [res, self.label]
		
	def get_index(self, im) :
		"""Returns the index in the list for the right image, or False
		if the index doesn't exists or if it is out of the lifespan of the poly"""
		"""i = None
		if im < self.start or im>self.stop :
			i=-1
		elif im in self.key_frames :
			i=self.key_frames.index(im)
		return i"""
			
	def get_rect(self, im, force_regenerate=False, with_reference=False) :
		"""Returns the rectangle associated with the frame"""
		#if self.key_poly != [] :
		if self.polygon.key_frames != [] :
			i=self.get_index(im)
			if i==None or force_regenerate or with_reference :
				if with_reference :
					pol = self.get_reference_poly(im)
				else :
					pol=self.get_poly(im)
				r= [pol[0][0], pol[0][1], 2,2]
				for point in pol :
					if point[0] < r[0] :
						r[2] = r[2] + r[0] - point[0]
						r[0] = point[0]
					elif point[0]> r[0]+r[2] :
						r[2] = point[0] - r[0]
					if point[1] < r[1] :
						r[3] = r[1]+r[3]-point[1]
						r[1] = point[1]
					elif point[1]> r[1]+r[3] :
						r[3] = point[1] - r[1]
				if force_regenerate :	# insert it
					#print [i, True]	# Debug
					self.rectangle.add_object(r, im)
					####self.rect[i] = r	# If its returns an error, check if i is defined
					
			elif i != -1 :
				r = self.rectangle.get_object(im)
				####r = self.rect[i]				
		else :
			r = [0,0,1,1]
			print "Calling an undefined rectangle... strange"
		"""rr = []
		for p in r :
			rr.append(int(p))"""
		return list(r)	
				
	def get_poly(self, im) :
		"""Function that returns the polygon corresponding to the image"""
		p = self.polygon.get_object(im)
		"""i=self.get_index(im)
		if self.key_frames == [] :
			print 'no key frames'
			pass
		elif im < self.key_frames[0] :
			p = self.key_poly[0]
			#print "TOO EARLY"	# Debug
		elif im > self.key_frames[-1] :
			p = self.key_poly[-1]
			#print "TOO LATE"	# Debug
		elif i==None :
			j=0
			while j<len(self.key_frames) and self.key_frames[j] < im :
				j+=1	# Gives the superior bound
			p = self.interpolate(j, im)
		else :
			p = self.key_poly[i]"""
		return p
		
	def interpolate(self, next_idx, t) :
		"""Function that returns the linearily interpolated polygon"""
		"""t1 = self.key_frames[next_idx-1]	# Calculate time delta
		t2 = self.key_frames[next_idx]
		x = float(t-t1)/(t2-t1)		# Weighted average
		p = []
		for pp in range(len(self.key_poly[next_idx])) :
			p1 = self.key_poly[next_idx-1][pp]	# Extract points
			p2 = self.key_poly[next_idx][pp]
			pt = [-1,-1]
			pt[0] = int( p1[0] + (p2[0]-p1[0])*x )	# Linear interpolation
			pt[1] = int( p1[1] + (p2[1]-p1[1])*x )
			p.append(pt)
		return p"""	
		
	def draw_lines(self, image, i, draw_reference=False) :
		"""Draws a line, or a polygon, depending on the closed argument"""
		if draw_reference :
			p = self.get_reference_poly(i)
			col = self.r_color
		else :
			p=self.get_poly(i)
			col = self.color
		if len(p) >=2 :
			#print p
			pp = []
			for j in p :
				pp.append((int(j[0]),int(j[1])))
			self.cv.PolyLine(image, [pp], self.close, col, 1)
		return image
		
	def draw_center(self, image, i) :
		"""Draws the center of a rectangle"""
		c = self.get_center(i)
		#print 'drawing circle'
		self.cv.Circle(image, c, 5, self.color, -1)
		return image
		
	def draw_points(self, image, i) :
		"""Function that draws all points on the screen
		Should be called after drawing the lines"""
		p=self.get_poly(i)
		for point in p :
			#print point
			self.cv.Circle(image, tuple(point), 3, self.color, -1)
		
		if self.big_points != [] :
			#print p
			self.cv.Circle(image, tuple(p[0]), 8, p[1], -1)
		if not self.moving :
			self.big_points = []
		return image
	def draw_label(self, image, i) :
		p = self.get_poly(i)
		"""Function that adds a label with the number of the polygon"""
		rect = (p[0][0],p[0][1],30,20)
		self.cv.PutText(image, str(self.label), (p[0][0], p[0][1]), self.font, self.color)
		return image	
	def draw_polygon(self, image, i, cv, debug=False) :
		"""Functions that put all together"""
		self.cv = cv
		if not self.font_init :
			font = cv.InitFont(cv.CV_FONT_HERSHEY_PLAIN, 1, 1) 	# Initializing font
			self.font = font
			self.font_init = True
		else :
			font = self.font
		#print (self.deleted, [i,self.start, self.stop])
		if not self.deleted and i >= self.start and i <= self.stop :
			if self.get_poly(i) != []:
				image=self.draw_lines(image, i, draw_reference=False)
				image=self.draw_lines(image, i, draw_reference=True)
				image=self.draw_points(image, i)
				image=self.draw_label(image, i)
				if debug :
					print '---'
					print self.get_rect(i, True)
					r=self.get_rect(i, True)
					p=()
					self.cv.Rectangle(image, (r[0],r[1]), (r[0]+r[2],r[1]+r[3]), (100, 100, 100), 6)
			#elif self.center_rect != (-1,-1) :
			#	c = self.center_rect
			#	self.cv.Circle(image, c, 5, self.color, -1)
		return image
	def point_in_poly(self, tup, i, with_reference = False) :
		if with_reference :
			p = self.get_reference_poly(i)
		else :
			p = self.get_poly(i)
		r = self.cv.PointPolygonTest(p, tup, 1)
		if r>0 :
			r = True
		else :
			r = False
		return r
		
	def is_all_in_screen(self, im) :
		"""Returns 'True' if the cell is totally contained in the image
		
		:returns: Returns 'True' if the cell (and its reference) is totally contained in the
			image
		:rtype: bool"""
		#r = self.get_rect(im)
		r = self.rectangle.get_object(im)
		s = self.movie.get_size()
		#print r, 'r'
		out = True
		if not (r[0] >= 0 and r[1] >= 0 and r[0]+r[2] < s[0] and r[1]+r[3] < s[1]) :
			out = False
		return out
		
class Cells :
	"""Class that manages polygons"""
	def __init__(self, movie, use_gtk=True) :
		self.movie = movie
		self.poly_list = []	# The polygone list (these are the correct indexes for labelling)
		self.label = -1
		self.cell_wrappers = self._load_cell_wrappers()
	def init(self) :
		"""Second initiation"""
		self.nb_im = self.movie.get_nb_frames()
	def _load_cell_wrappers(self) :
		"""Function that returns a dict of cell wrapper modules"""
		wrappers = {}
		for t in list_wrappers :
			m = imp.load_source(t[0], t[1])
			wrappers[m.get_name()] = m
		return wrappers	
	def get_save_dict(self) :
		ce_l = self.get_cells_list()
		l = []
		for ce in ce_l :
			dic = {}
			index = ce.get_label()
			save_dic = ce.get_save_dict()
			dic['label'] = {'desc' : 'label (ex-index) of the cell', 'value' : index}
			dic['type'] = {'desc' : 'wrapper to use', 'value' : ce.type}
			dic['cell_data'] = {'desc' : 'The internal parameters of the concerned cell', 'value' : save_dic}
			l.append(dic)
		out = {}
		out['cells'] = {'desc' : "A list of all the initiated cells, with their prameters", 'value' : l}
		return out
	def set_save_dict(self, dic) :
		print "-- Loading cell informations"
		ce_l = dic['cells']['value']
		self.init()
		for ce in ce_l :
			wrapper_type = ce['type']['value']
			c = self.new_polygon(wrapper_type, gtk=True)
			self.add_polygon(c)
		for ce in ce_l :
			self.get_cells_list()[ce['label']['value']].set_save_dict(ce['cell_data']['value'])
		print "-- Done"
	def get_cells_list(self) :
		return self.poly_list
	def get_cell_wrappers_dict(self) :
		return self.cell_wrappers
		
	def new_polygon(self, wrapper_name, gtk=False) :
		"""Function that returns a Polygon object
		this polygon is not integrated in the polygons_list.
		Run add_polygon to do so"""
		self.label +=1
		inst = self.get_cell_wrappers_dict()[wrapper_name].CellWrapper(self.movie, self.nb_im, self.label, gtk)
		return inst
	
	def add_polygon(self, poly) :
		print len(self.poly_list)
		poly.set_label(len(self.poly_list))
		self.poly_list.append(poly)
		
	def draw_all(self, image, im, cv, debug=False) :
		for i in self.poly_list :
			if not i.deleted :
				image = i.draw_polygon(image, im, cv, debug)
		return image
	def delete_poly(self, i) :
		"""Set the deleted flag on apolygon, given its index in the poly_list"""
		self.poly_list[i].deleted = True
		self.poly_list[i].close = True
	def clear_cells_list(self, poly_type=None) :
		for i in range(len(self.poly_list)) :
			if poly_type == None or self.poly_list[i].type == poly_type:
				self.delete_poly(i)
			
