# Modules that manages the definition of cells through a moving
#+ polygons. It interfaces the GUI and the ct_cell.Cell functions
# It derives from CellTracking_pygame by MW, July 2012, GPLv3+
import cv, os, time, numpy
#import poly_tmp


class PolygonsInterface :
	def __init__(self, movie) :
		#self.polygons = poly_tmp.Polygons(movie)
		self.movie = movie
		#self.polygons = self.movie.cells2
		self.polygons = self.movie.cells
		self.panel_visible = False
		self.panel_process_visible = False
		self.active = False
		self.keyboard_visible = False
		
		# TMP vars?
		self.move = False
		self.polygon_mode = 'polygon'
		self.debug = False
		self.current_polygon = []
		self.select_i = None			# The index of the selected polygons on the screen
		self.close = True	# To stop playing if a polygon is being drawn
		self.hotkeys = "==== Color code ====\n\
green = new polygon\n\
blue  = selected polygon\n\
red   = unselected polygon\n\n\
==== Key bindings ====\n\
c     = switch between polygon/rectangle mode\n\
s     = end the selected polygon\n\
z     = delete selected polygon (or the last created)\n\
d     = enters/quit debug mode\n\
h     = show this help"
	def init(self) :
		"""This init is called after the movie is properly instanciated"""
		self.nb_im = self.movie.get_nb_frames()
		self.polygons.init()
	def add_gtk_instance(self, gtk, gobject=None) :
		self.gobject = gobject
		self.gtk = gtk
	def add_buttons(self) :
		l1 = self._bouton(self.gtk, 'Manage cells', self._show_polygons_panel, {})
		self.movie.display_m.add_button(l1)
		l2 = self._bouton(self.gtk, 'Process cells', self._draw_process_cells, {})
		self.movie.display_m.add_button(l2)		
		l3 = self._bouton(self.gtk, 'Hide/Show cells', self._hide_show_cells, {})
		self.movie.display_m.add_button(l3)		

	def _draw_process_cells(self, w, data=None) :
		"""Draws the process grid window"""
		if not self.panel_process_visible :
			movie=self.movie
			movie_gtk = self.movie.display_m
			self.movie_gtk = movie_gtk
			gtk = self.gtk
			#movie_gtk.b_set_grid.set_sensitive(False)
			self.movie_gtk = movie_gtk
			if self.panel_visible :
				self._show_polygons_panel('')
			self.widgets_process = gtk.glade.XML('./modules/modules_core/grid.glade',"ProcessGrid")
			events = { '_close' : self._close_process,
					'process_ok' : self._process_ok,
					'_process_select_all' : self._process_select_all, 
					'_process_select_none' : self._process_select_none,
					'_process_export_selected' : self._process_export_selected,
					'_process_find_phases_selected' : self._process_find_phases_selected,
					'_process_extract_fluo_selected' : self._process_extract_fluo_selected}
			self.widgets_process.signal_autoconnect(events)	
			self.window_process = self.widgets_process.get_widget('ProcessGrid')
			self.window_process.set_title(movie.get_name())				# Set title
			self.scroll_process = self.widgets_process.get_widget('main_frame')
			self.box_process = gtk.VBox()
			self.box_process.show()
			self.scroll_process.add_with_viewport(self.box_process)
			
			# Create frame
			#for c in self.movie.cells2.get_cells_list() :
			for c in self.movie.cells.get_cells_list() :
				if not c.deleted :
					frame = c.gui.new_frame(self.gtk)
					frame.show()
					self.box_process.add(frame)
			
			#self.timer_process = self.gobject.timeout_add(200, self._update_cells, {'gtk' : gtk, 'movie' : movie, 'cells' : movie.cells2}) # call every 150ms
			self.timer_process = self.gobject.timeout_add(200, self._update_cells, {'gtk' : gtk, 'movie' : movie, 'cells' : movie.cells}) # call every 150ms
			self.panel_process_visible = True
		else :
			self._close_process()
	def _hide_show_cells(self, w, data=None) :
		self.active = not self.active
	def _close_process(self, source=None, event=None) :
		self.gobject.source_remove(self.timer_process)
		self.window_process.destroy()
		self.panel_process_visible = False
	def _process_ok(self, source=None, event=None) :
		"""useless?"""
		self._close_process()
	def _process_select_all(self, w) :
		#l_cells = self.movie.cells2.get_cells_list()
		l_cells = self.movie.cells.get_cells_list()
		for c in l_cells :
			if not c.deleted :
				c.select(True)
	def _process_select_none(self, w) :
		#l_cells = self.movie.cells2.get_cells_list()
		l_cells = self.movie.cells.get_cells_list()
		for c in l_cells :
			c.select(False)

	def _update_cells(self, d) :
		gtk = d['gtk']
		cells = d['cells']
		#for c in cells.get_cells_list() :
		#for c in self.movie.cells2.get_cells_list() :
		for c in self.movie.cells.get_cells_list() :
			if not c.deleted :
				# Updating phase
				c.set_frame(d['movie'].get_frame_index())
				c.gui.update_panel()
		return True
	
	def _process_export_selected(self, w) :
		# Show fileselect dialog
		gtk = self.gtk
		chooser = gtk.FileChooserDialog(title='Choose export folder',action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		response = chooser.run()
		r = False
		if response == gtk.RESPONSE_OK:
			exp = chooser.get_filename()+'/export-'+str(self.movie.get_name()) + '/'
			if not os.path.isdir(exp) :
				os.mkdir(exp)
			r = True
		elif response == gtk.RESPONSE_CANCEL:
			pass
		chooser.destroy()
		if r :
			#l_cells = self.movie.cells2.get_cells_list()
			l_cells = self.movie.cells.get_cells_list()
			for c in l_cells :
				if c.is_selected() :
					print "-- Cell %s" % str(c.get_label())
					t = time.strftime("%d-%m-%Y--%Hh%M", time.localtime())
					folder = exp + str(self.movie.channels.get_association_names_table()[self.movie.get_view()[0]]) + '-' + str(c.get_label()) + '--' + t + '/'
					os.mkdir(folder)
					print "- Computing size"
					l_size = []
					for i in range(self.movie.get_nb_frames()) :
						if i % 100 == 0 :
							print "image %s" % str(i)
						p=c.get_reference_poly(i)
						p=numpy.array(p)
						size = p.max(axis=0)-p.min(axis=0)
						l_size.append(size)
					l_size = numpy.array(l_size)
					rect = l_size.max(axis=0)
					
					print '- Saving'
					
					# Data file
					txt = 'Save file : %s\nCell index : %s' % (str(self.movie.import_export.file), str(c.get_label()))
					f = open(folder + 'data.txt', 'w')
					f.write(txt)
					f.close()
					
					for i in range(self.movie.get_nb_frames()) :
						if i % 20 == 0 :
							print "image %s" % str(i)
						path = folder + 'image'+str(i) + '.png'
						path_cm = folder + 'image_cm'+str(i) + '.png'
						path_rm = folder + 'image_rm'+str(i) + '.png'
						c.export_frame(i, (path, path_cm, path_rm), rect=rect)
	def _process_find_phases_selected(self, w) :
		#l_cells = self.movie.cells2.get_cells_list()
		l_cells = self.movie.cells.get_cells_list()
		print 'Finding phases'
		for c in l_cells :
			if c.is_selected() :
				c.find_phases(warning = False)
	def _process_extract_fluo_selected(self, w, data=None) :
		# Show fileselect dialog
		gtk = self.gtk
		#l_cells = self.movie.cells2.get_cells_list()
		l_cells = self.movie.cells.get_cells_list()
		for c in l_cells :
			if c.is_selected() :
				print "-- Cell %s" % str(c.get_label())
				c.fluorescence.extract_fluo2()

	
	def set_click(self, tup) :
		if self.active :
			#if not self.move :
			self.movie.set_play(False)
			self.new_point(tup, self.movie.get_frame_displayed(), self.polygon_mode)
			
			if self.select_i != None and tup != (-1, -1) :
				p = self.polygons.poly_list[self.select_i] # Get polygon object
				i = self.movie.get_frame_displayed()
				if p.point_in_poly(tup, i, with_reference=True) :
					self.move = p.move(tup, i)
			else :
				self.move = False
	def set_move(self, tup) :
		if self.active and self.select_i != None :
			self.movie.set_play(False)
			p = self.polygons.poly_list[self.select_i] # Get polygon object
			if self.move :
				self.move = p.move2(tup, self.movie.get_frame_displayed())
				
	def new_point(self, point, im, poly_mode) :
		"""Functions that receive all the new points
		it manages to create a new polygon or not depending if the previous
		one is already closed"""
		inter = [0, -1]
		new = False
		if self.current_polygon == [] and self.polygons.poly_list == []:			# if no polygon -> Create one
			#print "New polygon" + str(label)	# Debug
			#p= self.movie.cells.get_cell_wrappers_dict()[poly_mode].CellWrapper(self.nb_im, label)
			p= self.polygons.new_polygon(poly_mode, gtk=True)
			p.start = im
			self.current_polygon = p
			self.close = self.current_polygon.add_point(point, im)
			if self.close :
				self.polygons.add_polygon(p)		
				self.current_polygon = []
			new = True
			# There is a polygon
		else :						# if there are polygons, check the intersections between the point and the polygon(s)
			inter = self.intersect(point, im)	# Checks if the click is in a selection	
				
		#if not self.polygons.poly_list[-1].close and not new:		# if there is an open polygon ! draw it
		if not new and self.current_polygon != [] :		# if there is an open polygon ! draw it
			#self.close = self.polygons.poly_list[-1].add_point(point, im)
			self.close = self.current_polygon.add_point(point, im)
			if self.close :
				self.polygons.add_polygon(self.current_polygon)
				#self.polygons.poly_list.append(self.current_polygon)				### The append should only occur when closed
				self.current_polygon = []
			# It is not open
		elif inter[0] :					# if there is an intersection
			self.polygons.poly_list[inter[1]].select(1)
			#if self.polygons.select != inter[1] and self.select_i != None:
			if self.select_i != inter[1] and self.select_i != None:
				self.polygons.poly_list[self.select_i].select(0)
			self.select_i = inter[1]
		elif not new :						# if there is no intersection -> Create a new polygon
			##label = len(self.polygons.poly_list)
			#print "New polygon" + str(label)	# Debug
			p= self.polygons.new_polygon(poly_mode, gtk=True)
			##p= self.movie.cells.get_cell_wrappers_dict()[poly_mode].CellWrapper(self.nb_im, label)
			p.start = im
			self.current_polygon = p
			self.close = self.current_polygon.add_point(point, im)
			if self.close :
				self.polygons.poly_list.append(p)				### The append should only occur when closed
				self.current_polygon = []
		if not inter[0] and not new:
			if self.select_i != None :
				self.polygons.poly_list[self.select_i].select(0)
				self.select_i = None
	def intersect(self, point, im) :
		inter = [0,-1]
		i=0
		while inter[0]==0 and i<len(self.polygons.poly_list):
			inter=self.polygons.poly_list[i].intersection(point, im)
			i+=1
		return inter
	def _show_polygons_panel(self, w, data=None) :
		if not self.panel_visible :
			self.box1 = None
			if self.panel_process_visible :
				 self._draw_process_cells('')	# Closing process cells
			gtk = self.gtk
			self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	  
			self.window.connect("delete_event", self._close)

			self.window.set_title("Manage cells")
			self.window.set_border_width(0)

			box1 = gtk.VBox(False, 0)
			self.window.add(box1)
			box1.show()

			box2 = gtk.HBox(False, 0)
			box2.set_border_width(0)
			box1.pack_start(box2, True, True, 0)
			box2.show()

			button = gtk.RadioButton(None, "Grid")
			#button.set_sensitive(False)
			button.connect("toggled", self._change_radiobutton, "grid")
			box2.pack_start(button, True, True, 0)
			button.show()

			button = gtk.RadioButton(button, "Rectangles")
			button.connect("toggled", self._change_radiobutton, "rectangles")
			#button.set_active(True)
			box2.pack_start(button, True, True, 0)
			button.show()

			button = gtk.RadioButton(button, "Polygons")
			button.connect("toggled", self._change_radiobutton, "polygons")
			button.set_active(True)
			box2.pack_start(button, True, True, 0)
			button.show()

			separator = gtk.HSeparator()
			box1.pack_start(separator, False, True, 0)
			separator.show()
			
			#self.box2 = self._rect_pol_panel(gtk)	# Creating the main box
			self.mainbox = self._rect_pol_panel(gtk)	# Creating the main box
			box1.pack_start(self.mainbox, False, True, 0)
			self.box1 = box1
			
			separator = gtk.HSeparator()
			box1.pack_start(separator, False, True, 0)
			separator.show()

			button = gtk.Button("close")
			button.connect_object("clicked", self._close, self.window,None)
			box1.pack_start(button, True, True, 0)
			#button.set_flags(gtk.CAN_DEFAULT)
			#button.grab_default()
			button.show()
			self.window.show()

			self.panel_visible = True
			self.active = True
		else :
			self.window.destroy()
			self.panel_visible = False
			self.active = False

	def _rect_pol_panel(self,gtk) :
		box2 = gtk.VBox(False, 0)
		box2.set_border_width(0)
		box2.show()
		
		linebox1 = gtk.HBox()
		box2.pack_start(linebox1)
		linebox1.show()
		b_save = self._bouton(gtk, 'Set polygon end', self._end_poly, -1)
		linebox1.pack_start(b_save)
		b_del = self._bouton(gtk, 'Delete selected', self._del_select, -1)
		linebox1.pack_start(b_del)
		
		linebox2 = gtk.HBox()
		box2.pack_start(linebox2)
		linebox2.show()
		b_undo = self._bouton(gtk, 'Undo', self._undo, -1)
		linebox1.pack_start(b_undo)
		b_keyboard = self._bouton(gtk, 'Keyboard', self._show_keyboard, -1)
		linebox1.pack_start(b_keyboard)
		b_debug = self._bouton(gtk, 'Debug', self._debug, -1)
		linebox1.pack_start(b_debug)
		return box2	
	def overlay(self, image) :
		if self.active :
			i = self.movie.get_frame_displayed()
			image = self.polygons.draw_all(image, i, cv, self.debug)
			if self.current_polygon != [] :
				image = self.current_polygon.draw_polygon(image, i, cv, self.debug)
		return image
	def _change_radiobutton(self, widget, data=None):
		if self.box1 != None :
			self.box1.remove(self.mainbox)
		#print "%s was toggled %s" % (data, ("OFF", "ON")[widget.get_active()])
		if data=='rectangles' and widget.get_active() :
			self.polygon_mode = 'rectangle'
			if self.box1 != None :
				self.mainbox = self._rect_pol_panel(self.gtk)
				self.box1.pack_start(self.mainbox)
		elif data=='polygons' and widget.get_active() :
			self.polygon_mode = 'polygon'
			if self.box1 != None :
				self.mainbox = self._rect_pol_panel(self.gtk)
				self.box1.pack_start(self.mainbox)
		elif data=='grid' and widget.get_active() :
			self.polygon_mode = 'grid'
			if self.box1 != None :
				self.mainbox = self.movie.grid._grid_panel(self.gtk)
				self.box1.pack_start(self.mainbox)
		elif data=='grid' and not widget.get_active() :
			self.movie.grid._leave_grid_panel(self.gtk)
		if self.box1 != None :
			self.box1.reorder_child(self.mainbox, 2)
	def _close(self, w, data=None) :
		self.window.destroy()
		self.panel_visible = False
		self.active = False
	def _close_keyboard(self, w, data=None) :
		self.keyboard_window.destroy()
		self.keyboard_visible = False
	def _end_poly(self, w, data=None) :
		#if self.polygons.select != None :
		if self.select_i != None :
			#self.polygons.poly_list[self.polygons.select].stop = self.movie.get_frame_displayed()
			self.polygons.poly_list[self.select_i].stop = self.movie.get_frame_displayed()
			#print "Stop saved for polygon " + str(self.polygons.select)
			print "Stop saved for polygon " + str(self.select_i)
	def _show_keyboard(self, w, data=None) :
		if self.keyboard_visible :
			self.keyboard_window.destroy()
			self.keyboard_visible = False
		else :
			self.keyboard_window = self.gtk.Window(self.gtk.WINDOW_TOPLEVEL)
			self.keyboard_window.connect("delete_event", self._close_keyboard)
			self.keyboard_window.set_title("Key bindings :)")
			self.keyboard_window.show()
			l = self.gtk.Label(self.hotkeys)
			l.show()
			self.keyboard_window.add(l)
			self.keyboard_visible = True	
	def _del_select(self, w, data=None) :
		if self.current_polygon != [] :
			self.current_polygon = []
		else :
			self.delete_selected(self.movie.get_frame_displayed())
	def _undo(self, w, data=None) :
		if self.current_polygon != [] :
			self.current_polygon = []
		else :
			self.undo(self.movie.get_frame_displayed())
	def delete_selected(self, im) :
		#if time.time() - self.last_undo > self.time_threshold :
		if self.select_i != None :
			self.polygons.delete_poly(self.select_i)
		#self.last_undo = time.time()
	def undo(self, im) :
		"""Functions thats erases the last polygon"""
		if self.polygons.poly_list != [] :#and (time.time() - self.last_undo > self.time_threshold) :
			i = -1
			while self.polygons.poly_list[i].deleted == True and -1*i<len(self.polygons.poly_list):
				i -= 1
			self.polygons.delete_poly(i)
		#self.last_undo = time.time()
	def _debug(self, w, data=None) :
		self.debug = not self.debug	
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	 # On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()# This packs the button into the window (a GTK container).
		return bouton
	def _test(self, w, data=None) :
		print 'testons!'
class Object :
	"""A stupid object"""
