#-*-coding:utf-8-*-
class Phase :
	def __init__(self, cell, movie) :
		self.type = 'point'
		self.color = ['grey', 'red', 'green']
		self.require = None
		self.name = 'NEBD'
		self.s_name = 'nebd'
		self.labels = [('unk', 'Unknown'), ('nonebd','No NEBD'), ('nebd','NEBD')]
		
		#self.phase = [0]*movie.get_nb_frames()
		self.phase_interval = []
		self.cell = cell
		self.movie = movie
		self.current_phase = {'index' : None,
								'frame' : None,}
		self.initiated = False
		self.desc = 'NEBD tag'
	def get_save_dict(self) :
		dic = {}
		dic['name'] = {'desc' : 'Name of the phase', 'value' : self.name}
		dic['s_name'] = {'desc' : 'Short name of the phase', 'value' : self.s_name}
		dic['desc'] = {'desc' : 'Description of the phase', 'value' : self.desc}
		
		dic['initiated'] = {'desc' : 'if the phase is initiated (difference between nocell and unknown)', 'value' : self.initiated}
		dic['phase_interval'] = {'desc' : 'interval where the phase is positive (there is a cell)', 'value' : self.phase_interval}
		return dic
	def set_save_dict(self, dic) :
		self.initiated = dic['initiated']['value']
		self.phase_interval = dic['phase_interval']['value']
		
	def get_panel(self, gtk) :
		"""Returns a panel to be integrated in the feature panel for instance"""
		self.initiated = True
		self.box = gtk.HBox()
		b_new = self._bouton(gtk, 'New', self._new_phase, {})
		self.box.pack_start(b_new, False, False)
		b_set_frame = self._bouton(gtk, 'Set '+self.name, self._set_frame, {})
		self.box.pack_start(b_set_frame, False, False)
		b_delete = self._bouton(gtk, 'Delete', self._delete_phase, {})
		self.box.pack_start(b_delete, False, False)
		
		sep1 = gtk.VSeparator()
		sep1.show()
		self.box.pack_start(sep1)
		self.info_box = gtk.HBox()
		l_frame = gtk.Label('Frame')
		l_frame.show()
		self.info_box.pack_start(l_frame, False, False)
		self.e_frame = gtk.Entry()
		self.e_frame.show()
		self.e_frame.set_sensitive(False)
		self.e_frame.set_width_chars(4)
		self.info_box.pack_start(self.e_frame, False, False)
		self.info_box.show()
		self.box.pack_start(self.info_box)
		
		sep2 = gtk.VSeparator()
		sep2.show()
		self.box.pack_start(sep2)
		self.cb_period = gtk.combo_box_new_text()
		self._phase_list_update()		# Populating cb_period
		self.cb_period.connect('changed', self._change_period, {})
		self.cb_period.show()
		self.box.pack_start(self.cb_period,True, True)
		b_auto = gtk.CheckButton('new frame auto')
		self.auto_mode = False
		b_auto.set_active(False)
		b_auto.show()
		b_auto.connect("toggled", self._auto_new_frame)
		self.box.pack_start(b_auto, True, True)
		self._info_box_update()
		if self.current_phase['index'] != None :
			self.cb_period.set_active(self.current_phase['index'])
		return self.box
	
	def get_phase(self, i) :
		"""Returns a tuple (bool, shortname, name, color)"""
		if self.initiated :
			ph = 1
		else :
			ph = 0

		for interval in self.phase_interval :
			if i>=interval[0] and i<=interval[1] :
				ph = 2
				break
		#boolean = self.phase[i]
		return (ph, self.labels[ph][0], self.labels[ph][1], self.color[ph])
	def find_phase(self, i) :
		print "No algorithm implemented for the moment, sorry"
	def _label(self, frame) :
		return "Frame " + str(frame)
	def _info_box_update(self) :
		fr = ''
		if self.current_phase['frame'] != None :
			fr = str(self.current_phase['frame'])
		self.e_frame.set_text(fr)
	
	def _auto_save(self) :
		if self.auto_mode :
			if self._is_phase_ok(self.current_phase) :# Save the frame if correct
				self._save_phase(self.current_phase)
				self._new_phase('','')
		else :
			pass
	def _auto_new_frame(self, w, data=None) :
		self.auto_mode = w.get_active()
	def _new_phase(self, w, data=None) :
		"""This function creates a new period if the previous is finished or reset it
		if it is not"""
		# A different behavior must be implemented when editing instead of creating ???
		if self.auto_mode or not self._is_phase_ok(self.current_phase) :	# Or replace it
			self._set_default_phase()
		else : #if self._is_phase_ok(self.current_phase) :	# Save it as a new one
			#### The phase list must be sorted, don't forget
			self._save_phase(self.current_phase)
			self._set_default_phase()		# Create a new one
		self._info_box_update()
		self.cb_period.set_active(-1)	# Start with nothing selected
	def _save_phase(self, ph) :
		if ph['index'] == None :
			self.phase_interval.append(ph['frame'])	# Add it to list
		else :
			self.phase_interval[ph['index']] = ph['frame']
		self._phase_list_update()
		# Sorting will be implemented later...
		# Create list with original index [begin_frame, label, index]
		# Sort according to begin frame
		# Create correspondence list
	def _phase_list_update(self) :
		m = self.cb_period.get_model()	# Clean list
		for i in  m :
			self.cb_period.remove_text(0)
		for i in self.phase_interval :
			self.cb_period.append_text(self._label(i))
		
	def _is_phase_ok(self, ph) :
		ret = True
		if ph['frame']==None :
			ret = False
		return ret
	def _set_default_phase(self) :
		self.current_phase = {'index' : None,
								'frame' : None}
	def _set_frame(self, w, data=None) :
		i = self.movie.get_frame_index()
		self.current_phase['frame'] = i
		self._auto_save()
		self._info_box_update()

	def _delete_phase(self, w, data=None) :
		i = self.cb_period.get_active()
		if i != -1 :
			self.phase_interval.pop(i)
		self._info_box_update()
		self._phase_list_update()
		self._new_phase('','')
	def _change_period(self, w, data=None) :
		"""Selects a new period"""
		i = w.get_active()
		if i != -1 :
			ph = self.phase_interval[i]
			self.current_phase = {'frame' : ph, 'index' : i}
			self._info_box_update()
			
	def get_name(self) :
		return self.name
	def get_s_name(self) :
		return self.s_name
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	 # On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()# This packs the button into the window (a GTK container).
		return bouton
