#!/usr/bin/python
# -*- coding: utf-8 -*-
# this modules contains the required function to extract cellular cycle from the movie
# MW 2012 GPLv3+

import gobject, time, os
import numpy as np
import cv2
import scipy, Image #, pylab

class Fluorescence :
	"""This class is instanciated by a cell object,
	it contains the functions required to determine if the cell exist, 
	and in which part of the cell cycle it lives"""
	
	def __init__(self, cell, movie) :
		self.cell = cell
		self.movie = movie
		self.n = self.movie.get_nb_frames()
	
	def extract_fluo_GTK(self, cv, movie_gtk, warning=True, index=None, obj=None) :
		"""Functions that locks everything before running extract_fluo"""
		#print [index, obj]
		gtk = movie_gtk.gtk
		self.movie_gtk = movie_gtk
		self.movie.grid.hide_grid()			# Hide grid
		#movie_gtk.b_set_grid.set_sensitive(False)	# Blocking grid editing
		
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.connect("destroy", self._destroy_progress)

		table = gtk.Table(3,2, True)
		table.show()
		self.dialog.add(table)
		
		# Create the ProgressBar
		self.pbar = gtk.ProgressBar()
		table.attach(self.pbar, 0,2,1,2)
		self.pbar.show()
			
		
		if warning :
			self.dialog.set_title("Please check playback options")		
			label = gtk.Label("The current playback settings (speed, view mode)\n\
will be used to render the cell. Please correct them before clicking ok.\n\
Please note that for the moment the export runs from the beginning to the\n\
end of the video.")
			table.attach(label, 0, 2,0,1)
			b_ok = self._bouton(gtk, 'Ok', self._b_ok, {'cv' : cv, 'index' : index, 'obj' : obj} )
			table.attach(b_ok, 0,1,2,3)
			b_cancel = self._bouton(gtk, 'Cancel', self._destroy_progress, {})
			table.attach(b_cancel, 1,2,2,3)		
			self.pbar.set_text('Press Ok when ready')	
		else :
			self.dialog.set_title("Getting fluorescence")
			self._b_ok('',cv, index, obj)
			self.pbar.set_text('Waiting for other extractions to complete')

		self.first_run_progress = True
		self.running_extract = False
		self.dialog.show_all()
		self.frac = 0
	def extract_fluo2(self) :
		self._b_ok('')
	def _b_ok(self, w, data=None) :
		#cv = data['cv']
		#index = data['index']
		#obj = data['obj']
		#self.movie.set_locked(True)
		n = self.movie.get_nb_frames()
		
		out_text = {'simple_mean' : 'Frame, Sum (corrected by the reference stripe)\n\n', 
					'mean_noref' : 'Frame, Sum (no background correction) \n\n',
					'channel' : self.movie.channels.get_association_names_table()[self.movie.get_view()[0]]}
		# Second field useless now
		
		for i in range(n) :
			if i >= self.cell.start and i <= self.cell.stop :
				c = self.cell.get_cell(i, depth=8, col_channels=1, fmt='cv2')
				
				cel_tot = c[0]
				cel_mas = c[1]
				ref_mas = c[2]
				
				#cel = cel_tot[cel_mas!=0]
				#ref = cel_tot[ref_mas!=0]
				cel = np.ma.array(cel_tot, mask=(1-cel_mas.astype(bool)))
				ref = np.ma.array(cel_tot, mask=(1-ref_mas.astype(bool)))
				"""cv2.imwrite('./t/'+str(i) + '.png', c[0])
				cv2.imwrite('./t/'+str(i)+'_ref.png', np.ma.filled(ref, 0))
				cv2.imwrite('./t/'+str(i)+'_cel.png', np.ma.filled(cel, 0))"""
				
				
				mean_ref = cel.sum() - ref.mean()*cel.count()
				#print (np.min(cel_tot), np.max(cel_tot), cel.mean(), ref.mean()), [cel_tot.size, cel.count(), ref.count(), cel_tot.dtype]
				#if str(mean_ref) == '--' :	# DEBUG
				#	print cel_tot
					#print [str(mean_ref), cel.sum(), ref.mean(), cel.count()]	## DEBUG
				out_text['simple_mean']  += str(i) + ', ' + str(mean_ref) + '\n'
				out_text['mean_noref']  += str(i) + ', ' + str(cel.sum()) + '\n'
			else :
				out_text['simple_mean'] += str(i) + ', ' +'na\n' 
				out_text['mean_noref'] += str(i) + ', ' +'na\n' 
			if i % 100 == 0 :
				print "frame %s" % str(i)
			self.movie.set_frame_index(i)
			
		print 'end'
		self.movie.set_frame_index(0)
		# Exporting file
		p = './exports/quantification/'
		path = p + str(self.movie.get_name())+'/'
		t = time.strftime("%d-%m-%Y--%Hh%M", time.localtime())
		print "Saving in %s" % path
		if not os.path.isdir(path) :
			os.mkdir(path)
		save_file = "Save file : " + str(self.movie.import_export.file) + '\n'
		cell_index = 'Cell index :' + str(self.cell.get_label()) + '\n'
		filename_sq = str(out_text['channel']) + '-' + str(self.cell.get_label()) + '--squarediff-' + t
		filename_sg = str(out_text['channel']) + '-' + str(self.cell.get_label()) + '--mean_noref-' + t
		f_sq = open(path+filename_sq, 'w')
		f_sq.write(cell_index+save_file+out_text['simple_mean'])
		f_sq.close()
		f_sg = open(path+filename_sg, 'w')
		f_sg.write(cell_index+save_file+out_text['mean_noref'])
		f_sg.close()
			
			
		
		#self.timer = gobject.timeout_add(100, self.extract_fluo,cv, index, obj)
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# New button
		bouton.connect("clicked", function, dico)	# Connnection
		bouton.show()					# This packs the button into the window (a GTK container).
		return bouton 
		
	def _destroy_progress(self, w, data=None) :
		#self.movie_gtk.b_set_grid.set_sensitive(True)
		self.dialog.destroy()
		
	def extract_fluo(self, cv, index, obj) :
		"""Function that find phases for each image
		it can be called by a timer, which explains the particular structure"""
		if self.first_run_progress and obj.index==index :
			self.running_extract = True
			self.frac = 0
			self.movie.set_speed(1)				# Speed 1
			self.movie.set_mode('manual')		# Mode manual
			self.movie.set_frame_index(0)		# Set frame index
			self.movie.grid.movie_gtk.lock()	# Lock
			self.movie.grid.hide_grid()	# Lock
			self.first_run_progress = False
			self.progress_data = {'i' : 0}
			self.pbar.set_text('')
			self.out_text = {'simple_mean' : 'Frame, Sum over the square (corrected by the reference stripe)\n\n', 
							'segment_mean' : 'Frame, Sum over the segmentation based on variance (corrected by the outside of the segmented cell)\n\n',
							'channel' : self.movie.channels.get_association_names_table()[self.movie.get_view()[0]]}
		
		ret = True
		if self.running_extract :
			i = self.progress_data['i']
			
			if i < self.n and obj.index==index :
				f = self.movie.get_frame_displayed()	# The frame currently displayed by opencv
				if f == i  :
					#print (index, obj)
					fc = self.cell.get_cell(out_mode=32)		# Getting OpenCV image
					fc_mask = self.cell.get_cell(channel='gaussian_MM', format='numpy')
					c_mask = self.cell.get_cell_without_reference_NP(fc_mask)	# Formatted as a an array, you should exclude 0 values before processing 
					c = self.cell.get_cell_without_reference(fc)	# Formatted as a an array, you should exclude 0 values before processing 
					r = self.cell.get_cell_reference(fc)		# idem Gets it without reference
					r_mask = self.cell.get_cell_reference_NP(fc_mask)		# idem Gets it without reference
					
					ref = (r*(np.ones(r_mask.shape)-r_mask.astype(bool))).astype('float32')
					cel = (c*c_mask.astype(bool)).astype('float32')
					
					cel=  cel.astype('uint16')
					ref=  ref.astype('uint16')
					r=  r.astype('uint16')
					c=  c.astype('uint16')
						
					mean_noref = np.sum(c) - np.mean(r[r!=0])*c.shape[0]*c.shape[1]
					#mean_noref = np.mean(c)# - np.mean(r[r!=0])*c.shape[0]*c.shape[1]
					
					mean_ref = np.sum(cel) - np.mean(ref[ref!=0])*cel[cel!=0].size#/n
					self.out_text['simple_mean']  += str(i) + ', ' + str(mean_noref) + '\n'
					self.out_text['segment_mean'] += str(i) + ', ' + str(mean_ref) + '\n'
					#print mean_noref
					
					# Save images
					#pylab.imshow(ref)
					#pylab.show()
					#print np.min(c)
					#print np.max(c)
					#scipy.misc.imsave('./tmp/'+str(i)+'_mref.png', (ref/256).astype('uint8'))		# Back to 8 bits
					#scipy.misc.imsave('./tmp/'+str(i)+'_ref.png', (r/256).astype('uint8'))
					#scipy.misc.imsave('./tmp/'+str(i)+'_mcel.png', (cel/256).astype('uint8'))
					#scipy.misc.imsave('./tmp/'+str(i)+'_cel.png', (c/256).astype('uint8'))

					# Save mean
					#print "----"
					#print ref.dtype
					#print cel.dtype
					#print c.dtype
					#print r.dtype
					
					# Playback
					self.movie.manual_next_frame = True
					self.movie.next_frame()
					if i%20 == 0 :
						print 'image '+str(i)
					self.frac = float(i+1)/self.n
					self.pbar.set_fraction(self.frac)
					i+=1
					
				elif f>i :
					print 'Gone too far, going back'
					self.movie.set_frame_index(i)
			else :
				ret = False
				print 'end'
				# Exporting file
				p = './exports/quantification/'
				path = p + str(self.movie.get_name())+'/'
				t = time.strftime("%d-%m-%Y--%Hh%M", time.localtime())
				if not os.path.isdir(path) :
					os.mkdir(path)
				save_file = "Save file : " + str(self.movie.import_export.file) + '\n'
				cell_index = 'Cell index :' + str(self.cell.get_index()) + '\n'
				filename_sq = str(self.out_text['channel']) + '-' + str(self.cell.get_index()) + '--squarediff-' + t
				filename_sg = str(self.out_text['channel']) + '-' + str(self.cell.get_index()) + '--segmentdiff-' + t
				f_sq = open(path+filename_sq, 'w')
				f_sq.write(cell_index+save_file+self.out_text['simple_mean'])
				f_sq.close()
				f_sg = open(path+filename_sg, 'w')
				f_sg.write(cell_index+save_file+self.out_text['segment_mean'])
				f_sg.close()
				
				
				# Closing everything
				self.pbar.set_fraction(0)
				self.movie.grid.movie_gtk.unlock()	# Unlock
				self._destroy_progress('', '')
				gobject.source_remove(self.timer)
				obj.index += 1			# Exporting next frame
			self.progress_data['i'] = i
		return ret
